#****************************************************************************
#**
#**  File     :  URA0106_script.lua
#**  Author(s):  Resin_Smoker, Optimus Prime
#**
#**  Summary  :  Cybran Mosquito, Drone Gunship Script
#**
#**  Special Thanks to :  ChirmayaWrongEmail, Eni, Neruz, Goom, Gilbot-X, Sorian
#**
#**  Copyright  2008
#****************************************************************************

#### Localy imported files ####
local CAirUnit = import('/lua/cybranunits.lua').CAirUnit
local EffectUtil = import('/lua/EffectUtilities.lua')
local util = import('/lua/utilities.lua')

#### Weapon local files ####
local CAAAutocannon = import('/lua/cybranweapons.lua').CAAAutocannon
local AIFBombGravitonWeapon = import('/lua/aeonweapons.lua').AIFBombGravitonWeapon

URA0106 = Class(CAirUnit) {

    Weapons = {
        MainGun = Class(CAAAutocannon) {
            CreateProjectileAtMuzzle = function(self, muzzle)
                ### Creates a number of extra projectiles at muzzle equal to numProjectiles
                numProjectiles = 2
                for i = 0, (numProjectiles -1) do
                    local proj = CAAAutocannon.CreateProjectileAtMuzzle(self, muzzle)
                end
            end,

           OnWeaponFired = function(self)
               if not self.unit:IsDead() and self:GetCurrentTarget() then
                  ###Sets the drone max speed to that of the target to help prevent overshoot.
                  ###Only activates for Air units that are not also experimental units.
                  self:ForkThread(self.unit.AirTargetCheck)
                  CAAAutocannon.OnWeaponFired(self)
               end
           end,

           OnLostTarget = function(self, parent)
               if not self.unit:IsDead() then
                   ### Resets globals and unit speed back to cruise values when target is lost
                   self.unit:SetSpeedMult(0.8)
                 self.unit:SetAccMult(0.8)
                   self.unit:SetTurnMult(0.125)
                   self.MyTarget = {}
                   self.MyTargetSpeed = nil
                   self.DistanceToTarget = nil
                   self.unit:GuardCarrier()
               end
               CAAAutocannon.OnLostTarget(self, parent)
           end,
        },
        Bomb = Class(AIFBombGravitonWeapon) {},
    },

### File pathing and special paramiters called ###########################

    ### Setsup parent call backs between drone and parent
    Parent = nil,

    SetParent = function(self, parent, droneName)
        self.Parent = parent
        self.Drone = droneName
    end,

    ### Thrust and exhaust effect pathing
    ExhaustLaunch = '/effects/emitters/missile_smoke_exhaust_02_emit.bp',
    BeamLaunch = '/effects/emitters/missile_exhaust_fire_beam_01_emit.bp',
    BeamCruise = '/effects/emitters/aeon_nuke_exhaust_beam_02_emit.bp',
    BeamTopspeed = '/effects/emitters/aeon_nuke_exhaust_beam_01_emit.bp',

##########################################################################

    OnCreate = function(self, builder, layer)
    CAirUnit.OnCreate(self,builder,layer)
        ### Are we dead?
        if not self:IsDead() then
 
            ### Start of launch special effects
            self:ForkThread(self.LaunchEffects)

            ### Global Varibles###
            self.BeamExhaustEffectsBag = {}
            self.LaunchExhaustEffectsBag = {}
            self.DroneMaxSpeed = 25
            self.DistanceToTarget = nil
            self.MyTarget = {}
        end
    end,

    LaunchEffects = function(self)
        ### Are we dead?
        if not self:IsDead() then

            ### Launch Sound effect
            self:PlayUnitSound('Launch')

            ### Makes the drone launch fast and strait
            self:SetSpeedMult(2.0)
          self:SetAccMult(1.0)
            self:SetTurnMult(0.0001)
   
            ### Attaches effects to drone during launch
            table.insert(self.LaunchExhaustEffectsBag, CreateAttachedEmitter(self, 'contrail', self:GetArmy(), self.ExhaustLaunch):ScaleEmitter(0.5))
            table.insert(self.LaunchExhaustEffectsBag, CreateBeamEmitterOnEntity(self, 'exhaust', self:GetArmy(), self.BeamLaunch))

            ### Duration of launch
            WaitSeconds(2)

            ### Launch effect clean up
            if not self:IsDead() then
                EffectUtil.CleanupEffectBag(self,'LaunchExhaustEffectsBag')

                ### Resets the speed and turn rates after launch
                   self:SetSpeedMult(1.0)
                 self:SetAccMult(0.8)
                   self:SetTurnMult(0.125)
               
                ### Sets cruise thrust effect after launch
                table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine01', self:GetArmy(), self.BeamCruise))
                table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine02', self:GetArmy(), self.BeamCruise))
                table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine03', self:GetArmy(), self.BeamCruise))

                ### Heartbeat event to monitor the drones distance from the carrier, if the drone gets too far away it is recalled to the carrier
                self:HeartBeatDistanceCheck()
            end
        end
    end,

    HeartBeatDistanceCheck = function(self)
        ### Global variable setup
        self.AwayFromCarrier = false

        while self and not self:IsDead() and not self.Parent:IsDead() do
            WaitSeconds(2)
            if not self:IsDead() and not self.Parent:IsDead() then
                local dronePos = self:GetPosition()
                local parentPos = self.Parent:GetPosition()
                local distance = VDist2(dronePos[1], dronePos[3], parentPos[1], parentPos[3])
                if distance > 65 and self.AwayFromCarrier == false then
                    ### Disables weapons and returns drone to carrier if the drone is past a range of **65***
                    for i = 1, self:GetWeaponCount() do
                        local wep = self:GetWeapon(i)
                        wep:SetWeaponEnabled(false)
                        wep:AimManipulatorSetEnabled(false)
                        self:SetScriptBit('RULEUCC_RetaliateToggle', false)
                        IssueStop({self})
                        IssueClearCommands({self})
                    end
                    self.AwayFromCarrier = true
                    self:ForkThread(self.GuardCarrier)

                elseif distance <= 60 and self.AwayFromCarrier == true then
                    ### Enables weapons if the drone is in range of the carrier, allowing drone to engage a targets of opportunity
                    for i = 1, self:GetWeaponCount() do
                        local wep = self:GetWeapon(i)
                        wep:SetWeaponEnabled(true)
                        wep:AimManipulatorSetEnabled(true)
                       self:SetScriptBit('RULEUCC_RetaliateToggle', true)
                    end
                    self.AwayFromCarrier = false

                    ### Resets the speed and turn rates upon returning to the carrier
                    self:SetSpeedMult(1.0)
                  self:SetAccMult(0.8)
                    self:SetTurnMult(0.125)
                end
            end
        end
    end,

    OnMotionHorzEventChange = function(self, new, old)
        ### Should the drone stop flying it will automaticly be re-assigned to guard the carrier
        if not self:IsDead() and new == 'Stopped' and not self.Parent:IsDead() then
            ### Clears the current drone commands if any and forces the drone to guard the carrier         
            self:ForkThread(self.GuardCarrier)
            if self.BeamExhaustEffectsBag then
                ### Engine effect clean up
                EffectUtil.CleanupEffectBag(self,'BeamExhaustEffectsBag')
            end

        elseif not self:IsDead() and new == 'Stopping' and not self.Parent:IsDead() then
            ### Clears the current drone commands if any and forces the drone to guard the carrier           
            self:ForkThread(self.GuardCarrier)
            if self.BeamExhaustEffectsBag then
                ### Engine effect clean up
                EffectUtil.CleanupEffectBag(self,'BeamExhaustEffectsBag')
            end

        ###Enables cruise mode only if the drone has no target
        elseif new == 'Cruise' and not self:IsDead() and not self:GetTargetEntity() then
            if self.BeamExhaustEffectsBag then
                ### Engine effect clean up
                EffectUtil.CleanupEffectBag(self,'BeamExhaustEffectsBag')
            end
                table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine01', self:GetArmy(), self.BeamCruise))
                table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine02', self:GetArmy(), self.BeamCruise))
                table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine03', self:GetArmy(), self.BeamCruise))
                self:SetSpeedMult(1.0)
                self:SetAccMult(0.8)
                self:SetTurnMult(0.125)
        end               
        CAirUnit.OnMotionHorzEventChange(self, new, old)
    end,

    OnMotionTurnEventChange = function(self, newEvent, oldEvent)
        if not self:IsDead() then
            if newEvent == 'SharpTurn' then
                if self.BeamExhaustEffectsBag then
                    ### Engine effect clean up
                    EffectUtil.CleanupEffectBag(self,'BeamExhaustEffectsBag')
                end
                table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine01', self:GetArmy(), self.BeamCruise))
                table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine02', self:GetArmy(), self.BeamCruise))
                table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine03', self:GetArmy(), self.BeamCruise))
                if self:GetTargetEntity() and not self.OnWeaponFired then
                   self:ForkThread(self.unit.AirTargetCheck)
                end
            end
        end
        CAirUnit.OnMotionTurnEventChange(self, newEvent, oldEvent)
    end,

    AirTargetCheck = function(self)
        if not self.unit:IsDead() and self.unit:GetTargetEntity() then 
            self.MyTarget = self:GetCurrentTarget()
            if table.find(self.MyTarget:GetBlueprint().Categories,'AIR') and not table.find(self.MyTarget:GetBlueprint().Categories,'EXPERIMENTAL') then
                self:ForkThread(self.unit.FindTargetSpeed)
            else
                ### Should the current target not be an air target lower speed and increase the turning multi's to better engage non-air target
                self.unit:SetSpeedMult(0.6)
                self.unit:SetAccMult(0.8)
                self.unit:SetTurnMult(1.0)
            end
        end
    end,

    FindTargetSpeed = function(self)
        if not self.unit:IsDead() then
            self.MyTargetSpeed = self:GetCurrentTarget():GetBlueprint().Air.MaxAirspeed
        end
        self:ForkThread(self.unit.CalcDistanceToTarget)
    end,

    CalcDistanceToTarget = function(self)
        if not self.unit:IsDead() then
            local targetPos = self:GetCurrentTarget():GetPosition()
            local dronePos = self.unit:GetPosition()
            self.DistanceToTarget = VDist2(dronePos[1], dronePos[3], targetPos[1], targetPos[3])
        end
        if self.DistanceToTarget >= 20 then
            self:ForkThread(self.unit.Afterburner)
        else
            self:ForkThread(self.unit.MatchTargetSpeed)
        end
    end,

    Afterburner = function(self)
        if not self.unit:IsDead() then
            if self.BeamExhaustEffectsBag then
                ### Engine effect clean up
                EffectUtil.CleanupEffectBag(self,'BeamExhaustEffectsBag')
            end
            self.unit:SetSpeedMult(1.25)
          self.unit:SetAccMult(1.0)
            self.unit:SetTurnMult(0.25)
            table.insert(self.unit.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self.unit, 'engine01', self.unit:GetArmy(), self.unit.BeamTopspeed))
            table.insert(self.unit.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self.unit, 'engine02', self.unit:GetArmy(), self.unit.BeamTopspeed))
            table.insert(self.unit.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self.unit, 'engine03', self.unit:GetArmy(), self.unit.BeamTopspeed))
        end
    end,

    MatchTargetSpeed = function(self)
        if not self.unit:IsDead() then
            if self.BeamExhaustEffectsBag then
                ### Engine effect clean up
                EffectUtil.CleanupEffectBag(self,'BeamExhaustEffectsBag')
            end
            table.insert(self.unit.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self.unit, 'engine01', self.unit:GetArmy(), self.unit.BeamCruise))
            table.insert(self.unit.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self.unit, 'engine02', self.unit:GetArmy(), self.unit.BeamCruise))
            table.insert(self.unit.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self.unit, 'engine03', self.unit:GetArmy(), self.unit.BeamCruise))

            ###Sets the drone max speed to that of the target to help prevent overshoot.   
            self.unit:SetSpeedMult(self.MyTargetSpeed / self.unit.DroneMaxSpeed)
            self.unit:SetAccMult(1.0)
            self.unit:SetTurnMult(1.0)
        end
    end,

    GuardCarrier = function(self)
        if not self:IsDead() and not self.Parent:IsDead() then
            ### Tells the drone to guard the carrier
            IssueClearCommands(self)
            IssueGuard({self}, self.Parent)
            if self.BeamExhaustEffectsBag then
                ### Engine effect clean up
                EffectUtil.CleanupEffectBag(self,'BeamExhaustEffectsBag')
            end
            ### Increase all multi's to get back and guard the carrier quickly
            self:SetSpeedMult(1.0)
            self:SetAccMult(0.8)
            self:SetTurnMult(0.125)
            table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine01', self:GetArmy(), self.BeamCruise))
            table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine02', self:GetArmy(), self.BeamCruise))
            table.insert(self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity( self, 'engine03', self:GetArmy(), self.BeamCruise))
        end
    end,

    OnDamage = function(self, instigator, amount, vector, damagetype)
        CAirUnit.OnDamage(self, instigator, amount, vector, damagetype)
        if self:IsDead() == false and instigator and IsUnit(instigator) then
            IssueAttack({self}, instigator)
            self:ForkThread(self.unit.Afterburner)
        end
    end,

    OnKilled = function(self, instigator, type, overkillRatio)
        ### Disables weapons
        self:SetWeaponEnabledByLabel('MainGun', false)
        self:SetWeaponEnabledByLabel('Bomb', false)

        ### Clears the current drone commands if any
        IssueClearCommands(self)

        ### Engine effects clean up
        EffectUtil.CleanupEffectBag(self,'BeamExhaustEffectsBag')

        ### Notifies parent of drone death and clears the offending drone from the parents table
        if not self.Parent:IsDead() then
            self.Parent:NotifyOfDroneDeath(self.Drone)
            table.removeByValue(self.Parent.DroneTable, self)
            self.Parent = nil
        end

        ### Final command to finish off the drones death event
        CAirUnit.OnKilled(self, instigator, type, overkillRatio)
    end,

}

TypeClass = URA0106