#****************************************************************************
#**
#**  File     :  URA0106_script.lua
#**  Author(s):  Resin_Smoker
#**
#**  Summary  :  Cybran Mosquito, Drone Gunship Script
#**
#**  Copyright  2008
#****************************************************************************

#### Localy imported files ####
local CAirUnit = import('/lua/cybranunits.lua').CAirUnit
local EffectUtil = import('/lua/EffectUtilities.lua')

#### Weapon local files ####
local CAAAutocannon = import('/lua/cybranweapons.lua').CAAAutocannon
local AIFBombGravitonWeapon = import('/lua/aeonweapons.lua').AIFBombGravitonWeapon

URA0106 = Class(CAirUnit) {
    Weapons = {
        MainGun = Class(CAAAutocannon) {
            CreateProjectileAtMuzzle = function(self, muzzle)
                ### Creates a number of extra projectiles at muzzle equal to numProjectiles
                numProjectiles = 2
                for i = 0, (numProjectiles -1) do
                    local proj = CAAAutocannon.CreateProjectileAtMuzzle(self, muzzle)
                    ### Adds the extra weapon damage to shields
                    local data = self:GetBlueprint().DamageToShields
                    if proj and not proj:BeenDestroyed() then
                        proj:PassData(data)
                    end
                end
            end,
        },
        Bomb = Class(AIFBombGravitonWeapon) {},
    },

ExhaustBones = {
    'engine01',
    'engine02',
    'engine03',
    },
ContrailBones = {
    'contrail',
    },
AmbientExhaustBones = {
    'contrail',
    },
AmbientExhaustEffects = {
    '/effects/emitters/missile_smoke_exhaust_02_emit.bp',
    },	

    OnCreate = function(self, builder, layer)
    CAirUnit.OnCreate(self,builder,layer)
        ### Are we dead?
        if not self:IsDead() then
            ### Makes the drone launch fast and strait
            self:SetSpeedMult(2)
   	    self:SetAccMult(2)
            self:ForkThread(self.LaunchEffects)
        end
    end,

    LaunchEffects = function(self)
        ### Adds a few simple laucn effects
        self.AmbientExhaustEffectsBag = {} 
        FxTrailScale = 1,

        ### Launch Sound effect
        self:PlayUnitSound('Launch')

        ### Are we dead?
        if not self:IsDead() then
	
            ### Attaches effects to drone during launch
            for kE, vE in self.AmbientExhaustEffects do
                for kB, vB in self.AmbientExhaustBones do
                    table.insert( self.AmbientExhaustEffectsBag, CreateAttachedEmitter(self, vB, self:GetArmy(), vE ))
                end
            end	

            ### Duration of launch
            WaitSeconds(1)

            ### Launch effect clean up
            EffectUtil.CleanupEffectBag(self,'AmbientExhaustEffectsBag')

            ### Resets the speed and turn rates after launch
            self:SetSpeedMult(1)
   	    self:SetAccMult(1)
        end
    end,

    Parent = nil,

    SetParent = function(self, parent, droneName)
        self.Parent = parent
        self.Drone = droneName
    end,

    OnKilled = function(self, instigator, type, overkillRatio)

        ### Disables weapons###
        self:SetWeaponEnabledByLabel('MainGun', false)

        ### Clears the current drone commands if any ###
        IssueClearCommands(self)

        ### Notifies parent of drone death ###
        if not self.Parent:IsDead() then 
            self.Parent:NotifyOfDroneDeath(self.Drone)
            self.Parent = nil 
        end

        ### Final command to finish off the drones death event###
        CAirUnit.OnKilled(self, instigator, type, overkillRatio)
    end,

}

TypeClass = URA0106