#****************************************************************************
#**
#**  File     :  URA0106_script.lua
#**  Author(s):  Resin_Smoker
#**
#**  Summary  :  Cybran Mosquito, Drone Gunship Script
#**
#**  Special Thanks to :  ChirmayaWrongEmail, Eni
#**
#**  Copyright  2008
#****************************************************************************

#### Localy imported files ####
local CAirUnit = import('/lua/cybranunits.lua').CAirUnit
local EffectUtil = import('/lua/EffectUtilities.lua')

#### Weapon local files ####
local CAAAutocannon = import('/lua/cybranweapons.lua').CAAAutocannon
local AIFBombGravitonWeapon = import('/lua/aeonweapons.lua').AIFBombGravitonWeapon

URA0106 = Class(CAirUnit) {
    Weapons = {
        MainGun = Class(CAAAutocannon) {
            CreateProjectileAtMuzzle = function(self, muzzle)
                ### Creates a number of extra projectiles at muzzle equal to numProjectiles
                numProjectiles = 2
                for i = 0, (numProjectiles -1) do
                    local proj = CAAAutocannon.CreateProjectileAtMuzzle(self, muzzle)
                end
            end,

           OnWeaponFired = function(self)
               CAAAutocannon.OnWeaponFired(self)
               local drone = self.unit
               ### Is the drone dead
               if not drone:IsDead() then
                   ### Disables the units cloak on fire
                   self.unit:CloakDisable()
               end
           end,

           OnLostTarget = function(self, parent)
               CAAAutocannon.OnLostTarget(self, parent)
               ### Sets up local variables
               local drone = self.unit
               ### Is the drone dead
               if not drone:IsDead() then
                   ### Enables the units cloak when target lost
                   self.unit:CloakEnable()
               end
           end,
        },
        Bomb = Class(AIFBombGravitonWeapon) {

           OnWeaponFired = function(self)
               AIFBombGravitonWeapon.OnWeaponFired(self)
               local drone = self.unit
               ### Is the drone dead
               if not drone:IsDead() then
                   ### Disables the units cloak on fire
                   self.unit:CloakDisable()
               end
           end,

           OnLostTarget = function(self, parent)
               AIFBombGravitonWeapon.OnLostTarget(self, parent)
               ### Sets up local variables
               local drone = self.unit
               ### Is the drone dead
               if not drone:IsDead() then
                   ### Enables the units cloak when target lost
                   self.unit:CloakEnable()
               end
           end,
        },
    },

ExhaustBones = {
    'engine01',
    'engine02',
    'engine03',
    },
ContrailBones = {
    'contrail',
    },
AmbientExhaustBones = {
    'contrail',
    },
AmbientExhaustEffects = {
    '/effects/emitters/missile_smoke_exhaust_02_emit.bp',
    },	

    OnCreate = function(self, builder, layer)
    CAirUnit.OnCreate(self,builder,layer)
        ### Are we dead?
        if not self:IsDead() then
            ### Makes the drone launch fast and strait
            self:SetSpeedMult(2)
   	    self:SetAccMult(2)
            self:ForkThread(self.LaunchEffects)
            self:SetScriptBit('RULEUTC_CloakToggle', false)
            self:SetMaintenanceConsumptionActive()
        end
    end,

    Parent = nil,

    SetParent = function(self, parent, droneName)
        self.Parent = parent
        self.Drone = droneName
    end,

    LaunchEffects = function(self)
        ### Adds a few simple laucn effects
        self.AmbientExhaustEffectsBag = {} 
        FxTrailScale = 0.5,

        ### Launch Sound effect
        self:PlayUnitSound('Launch')

        ### Are we dead?
        if not self:IsDead() then
	
            ### Attaches effects to drone during launch
            for kE, vE in self.AmbientExhaustEffects do
                for kB, vB in self.AmbientExhaustBones do
                    table.insert( self.AmbientExhaustEffectsBag, CreateAttachedEmitter(self, vB, self:GetArmy(), vE ))
                end
            end	

            ### Duration of launch
            WaitSeconds(1)

            ### Launch effect clean up
            if not self:IsDead() then
                EffectUtil.CleanupEffectBag(self,'AmbientExhaustEffectsBag')

                ### Resets the speed and turn rates after launch
                self:SetSpeedMult(1)
   	        self:SetAccMult(1)
            end
        end
    end,

    CloakEnable = function(self)
        if not self:IsDead() then
            self:SetScriptBit('RULEUTC_CloakToggle', false)
            self:SetMaintenanceConsumptionActive()
            self:RequestRefreshUI()
        end
    end,

    CloakDisable = function(self)
        if not self:IsDead() then
            self:SetScriptBit('RULEUTC_CloakToggle', true)
            self:SetMaintenanceConsumptionInactive()
            self:RequestRefreshUI()
        end
    end,

    GuardCarrier = function(self)
        if not self:IsDead() and not self.Parent:IsDead() then
            IssueClearCommands(self)
            IssueGuard({self}, self.Parent)
        end
    end,

    OnMotionHorzEventChange = function(self, new, old)
        ### Should the drone stop flying it will automaticly be re-assigned to guard the carrier
        if not self:IsDead() and not self.Parent:IsDead() then
            if new == 'Stopped' then
                self:CloakEnable()
                self:GuardCarrier()
            end
        end
        CAirUnit.OnMotionHorzEventChange(self, new, old)
    end,

    OnKilled = function(self, instigator, type, overkillRatio)

        ### Disables weapons
        self:SetWeaponEnabledByLabel('MainGun', false)

        ### Clears the current drone commands if any 
        IssueClearCommands(self)

        ### Notifies parent of drone death and clears the offending drone from the parents table

        if not self.Parent:IsDead() then
            self.Parent:NotifyOfDroneDeath(self.Drone)
            table.removeByValue(self.Parent.DroneTable, self)

            ### Issues an retaliate order to a surviving drone if any
            if self:IsDead() == true and IsUnit(instigator) then
                local tableSize = table.getsize (self.Parent.DroneTable)
                if tableSize >= 1 then
                    local ranDrone = Random(1,tableSize)
                    LOG('Retaliation') 
                    IssueClearCommands({self.Parent.DroneTable[ranDrone]})
                    IssueAttack({self.Parent.DroneTable[ranDrone]}, instigator)
                end
            end

            self.Parent = nil
        end 

        ### Final command to finish off the drones death event
        CAirUnit.OnKilled(self, instigator, type, overkillRatio)
    end,

}

TypeClass = URA0106