#****************************************************************************
#**
#**  File     :  URL0304_script.lua
#**  Author(s):  Resin_Smoker & Optimus_Prime
#**
#**  Summary  :  Cybran Vulcanizer Artillery Script
#**
#**  Copyright  2008 4th Dimension
#****************************************************************************

### Misc Lua called ###
local CLandUnit = import('/lua/cybranunits.lua').CLandUnit
local utilities = import('/lua/utilities.lua')
local RandomFloat = utilities.GetRandomFloat
local EffectTemplate = import('/lua/EffectTemplates.lua')
local explosion = import('/lua/defaultexplosions.lua')
local CreateDeathExplosion = explosion.CreateDefaultHitExplosionAtBone

### Weapon Local lua called ###
local CIFArtilleryWeapon = import('/lua/cybranweapons.lua').CIFArtilleryWeapon

URL0403 = Class(CLandUnit) {
   
    Weapons = {
       	QuadCannon = Class(CIFArtilleryWeapon) {
            FxMuzzleFlash = {
                '/effects/emitters/proton_artillery_muzzle_01_emit.bp',
             #   '/effects/emitters/proton_artillery_muzzle_02_emit.bp',               
                '/effects/emitters/proton_artillery_muzzle_03_emit.bp',
             #   '/effects/emitters/proton_artillery_muzzle_04_emit.bp',                
            },
       
            PlayFxWeaponPackSequence = function(self)
                if self.SpinManip then
                    self.SpinManip:SetTargetSpeed(0)
                end
                CIFArtilleryWeapon.PlayFxWeaponPackSequence(self)
            end,
            
            CreateProjectileAtMuzzle = function(self, muzzle) 
                ### Spins the barrels 90 degrees after each shot
                #CreateRotator(unit, bone, axis, [goal], [speed], [accel], [goalspeed])
                self.SpinManip = CreateRotator(self.unit, 'Rotator', 'z', self.unit.Zaxis, 125, 0, 125)
                self.unit.Trash:Add(self.SpinManip)

                ### Updates firing Randomness but only if RapidFiringState is false
                self.unit.CurrentAccuracy = self.unit.MyWeapon:GetFiringRandomness()
                #LOG('Current Accuracy: ', self.unit.CurrentAccuracy)
                if self.unit.RapidFiringState == false and self.unit.CurrentAccuracy >= 0.25 then
                    self.unit.MyWeapon:SetFiringRandomness(self.unit.CurrentAccuracy - 0.125)
                end

                ### Resets weapon acuracy if the target has changed
                if not self.unit:IsDead() and self.unit.MyWeapon:GetCurrentTarget() and self.unit.RapidFiringState == false then
                    self.unit.CurrentTarget = self.unit.MyWeapon:GetCurrentTarget():GetEntityId() 
                    if self.unit.OldTarget == nil then
                        #LOG('Old target is nil')
                        ### Inputs new values into table
                        self.unit.OldTarget = self.unit.CurrentTarget
                    elseif self.unit.OldTarget ~= self.unit.CurrentTarget then
                        self.unit.MyWeapon:SetFiringRandomness(self.unit.MyWeapon:GetBlueprint().FiringRandomness)
                        ### Inputs new target values into OldTarget
                        self.unit.OldTarget = self.unit.CurrentTarget
                    end
                end

                CIFArtilleryWeapon.CreateProjectileAtMuzzle(self, muzzle)
            end,

            OnLostTarget = function(self)
                ### Resets weapon acuracy if target is lost
                if not self.unit:IsDead() and self.SpinManip then
                    #LOG('Target is Lost') 
                    self.SpinManip:SetSpeed(0)       
                    self.unit.MyWeapon:SetFiringRandomness(self.unit.MyWeapon:GetBlueprint().FiringRandomness)
                end
                CIFArtilleryWeapon.OnLostTarget(self)
            end,
        },
    },

    OnStopBeingBuilt = function(self,builder,layer)
        CLandUnit.OnStopBeingBuilt(self)
        ### Global variable setup
        self.MyWeapon = self:GetWeaponByLabel('QuadCannon')
        self.CurrentAccuracy = nil
        self.CurrentTarget = nil
        self.OldTarget = nil
        self.RapidFiringState = true
        self.Zaxis = 90
        self.Moving = false
        self.TurretTable = {}

        ### Removes unused bones after the unit has been built
        self:HideBone('PlasmaTurret', true)

        ### Calls the needed threads
        self:ForkThread(self.SecondaryTurretSpawn)
        self:ForkThread(self.HeartBeatChecks)
    end,

    OnScriptBitSet = function(self, bit)
        ### Selected the LOW rate of fire and HIGH accuracy
        CLandUnit.OnScriptBitSet(self, bit)
        if bit == 1 then 
            self.RapidFiringState = false
            self.MyWeapon:SetFiringRandomness(self.MyWeapon:GetBlueprint().FiringRandomness)
            self.MyWeapon:ChangeRateOfFire(self.MyWeapon:GetBlueprint().RateOfFire * 0.25) 
        end
    end,

    OnScriptBitClear = function(self, bit)
        ### Selectes the HIGH rate of fire and LOW accuracy
        CLandUnit.OnScriptBitClear(self, bit)
        if bit == 1 then
            self.RapidFiringState = true 
            self.MyWeapon:SetFiringRandomness(self.MyWeapon:GetBlueprint().FiringRandomness)
            self.MyWeapon:ChangeRateOfFire(self.MyWeapon:GetBlueprint().RateOfFire) 
        end
    end,

    HeartBeatChecks = function(self)
        while self and not self:IsDead() do
            WaitSeconds(1)
            ### Checks to see if the Vulcanizer is moving
            if not self:IsDead() and self.Moving == true then 
                self.VulcPos = self:GetPosition()
                local radius = 5 
                ### Apply damage to the area under it if there is an enemy unit within the set radius  
                local targetlist = utilities.GetEnemyUnitsInSphere(self, self.VulcPos, radius)
                if table.getsize (targetlist) > 0 then
                    ### Apply damage
                    DamageRing(self, self.VulcPos, 0.1, 5, 10000, 'Normal', false)
                    ### Create ground decals
                    local orientation = utilities.GetRandomFloat( 0, 2 * math.pi )
                    CreateDecal(self.VulcPos, orientation, 'nuke_scorch_003_albedo', '', 'Albedo', 5, 5, 50, 0, self:GetArmy())
                end
            end
        end
    end,

    OnMotionHorzEventChange = function(self, new, old)
        ### Updates the gloabal variable for when the Vulcanizers movement changes and 
        ### alters the accuracy of the secondary turret for when the Vulcanizer is moving
        if not self:IsDead() and new == 'Stopped' then
            self.Moving = false
            self.TurretTable[1]:NotifyOfTurretStopped(self)
        elseif not self:IsDead() then
            self.Moving = true
            self.TurretTable[1]:NotifyOfTurretMoving(self)
        end              
        CLandUnit.OnMotionHorzEventChange(self, new, old)
    end,

    SecondaryTurretSpawn = function(self)
        ### Only spawns the turret only if the Vulcanizer unit is not dead
        if not self:IsDead() then 

            ### Sets up local Variables used and spawns a turret at the Vulcanizer Attach_Point location 
            local myOrientation = self:GetOrientation()
      
            ### Gets the current position of the Vulcanizer Attach_Point in the game world
            local location = self:GetPosition('Attach_Point')

            ### Creates our turret on the Attach_Point and directs the unit to face the same direction as its Vulcanizer unit
            local turret = CreateUnit('url0404', self:GetArmy(), location[1], location[2], location[3], myOrientation[1], myOrientation[2], myOrientation[3], myOrientation[4], 'Land') 

            ### Adds the newly created turret to the Vulcanizer turret table
            table.insert (self.TurretTable, turret)

            ### Sets the Vulcanizer unit as the turrets parent
            turret:SetParent(self, 'url0403')
            turret:SetCreator(self) 

            ### Attaches the Turret to the parent
            turret:AttachTo(self, 'Attach_Point') 
 
            ###Turret clean up scripts
            self.Trash:Add(turret)
        end
    end,

    DestructionEffectBones = {
        'PlasmaTurret','Turret','PlasmaTurretSleeve',
        'PlasmaTurretBarrel1','PlasmaTurretSleeve01', 
        'Rotator','PlasmaTurretBarrel2','Sleeve',
        'barrel11','barrel12','barrel13','barrel14',
        'barrel21','barrel22','barrel23','barrel24',
        'barrel31','barrel32','barrel33','barrel34',
    },

    OnKilled = function(self, instigator, type, overkillRatio)
        ### Kills off the secondary turret
        self.TurretTable[1]:DestroySecondaryTurret(self) 

        ### Unhides the prop weapon to provide a corpse for the secondary turret
        self:ShowBone('PlasmaTurret', true)

        ### Removes autocannon barrel spin effects
        if self.SpinManip then 
            self.unit.Trash:Add(self.SpinManip)
        end   
        CLandUnit.OnKilled(self, instigator, type, overkillRatio)
    end,

    CreateDamageEffects = function(self, bone, Army )
        for k, v in EffectTemplate.CEMPGrenadeHit01 do   
            CreateAttachedEmitter( self, bone, Army, v ):ScaleEmitter(1.0)
        end
    end,

    CreateExplosionDebris = function( self, bone, Army )
        for k, v in EffectTemplate.ExplosionEffectsSml01 do
            CreateAttachedEmitter( self, bone, Army, v ):ScaleEmitter(1.5)
        end
    end,
    
    CreateFirePlumes = function( self, Army, bones, yBoneOffset )
        ### Fire plume effects
        local basePosition = self:GetPosition()
        for k, vBone in bones do
            local position = self:GetPosition(vBone)
            local offset = utilities.GetDifferenceVector( position, basePosition )
            velocity = utilities.GetDirectionVector( position, basePosition ) 
            velocity.x = velocity.x + utilities.GetRandomFloat(-0.45, 0.45)
            velocity.z = velocity.z + utilities.GetRandomFloat(-0.45, 0.45)
            velocity.y = velocity.y + utilities.GetRandomFloat( 0.0, 0.45)
            local proj = self:CreateProjectile('/effects/entities/DestructionFirePlume01/DestructionFirePlume01_proj.bp', offset.x, offset.y + yBoneOffset, offset.z, velocity.x, velocity.y, velocity.z)
            proj:SetBallisticAcceleration(utilities.GetRandomFloat(-1, 1)):SetVelocity(utilities.GetRandomFloat(1, 2)):SetCollision(false)
            local emitter = CreateEmitterOnEntity(proj, Army, '/effects/emitters/destruction_explosion_fire_plume_01_emit.bp')
            local lifetime = utilities.GetRandomFloat( 10, 30 )
        end
    end,

    CreateAmmoCookOff = function( self, Army, bones, yBoneOffset )
        ### Fire plume effects
        local basePosition = self:GetPosition()
        for k, vBone in bones do
            local position = self:GetPosition(vBone)
            local offset = utilities.GetDifferenceVector( position, basePosition )
            velocity = utilities.GetDirectionVector( position, basePosition ) 
            velocity.x = velocity.x + utilities.GetRandomFloat(-0.45, 0.45)
            velocity.z = velocity.z + utilities.GetRandomFloat(-0.45, 0.45)
            velocity.y = velocity.y + utilities.GetRandomFloat( 0.0, 0.65)

            ### Ammo Cookoff projectiles and damage
            self.DamageData = {
                BallisticArc = 'RULEUBA_HighArc',
                UseGravity = true, 
                CollideFriendly = true, 
                DamageAmount = 1000, 
                DamageFriendly = true, 
                DamageRadius = 3, 
                DamageType = 'Normal',
                } 
            ammocookoff = self:CreateProjectile('/projectiles/TIFAntiMatterShells01/TIFAntiMatterShells01_proj.bp', offset.x, offset.y + yBoneOffset, offset.z, velocity.x, velocity.y, velocity.z)
            ### SetVelocity controls how far away the ammo will be thrown
            ammocookoff:SetVelocity(Random(5,10))  
            ammocookoff:SetLifetime(20) 
            ammocookoff:PassDamageData(self.DamageData)
            self.Trash:Add(ammocookoff)
        end
    end,

    InitialRandomExplosionsCybrans = function(self)
        local position = self:GetPosition()
        local numExplosions =  math.floor( table.getn( self.DestructionEffectBones ) * Random(0.4, 1.0))
        # Create small explosions effects all over
        for i = 0, numExplosions do
            local ranBone = utilities.GetRandomInt( 1, numExplosions )
            local duration = utilities.GetRandomFloat( 0.25, 0.5 )
            self:PlayUnitSound('AmmoCookOff')
            self:ShakeCamera(4, 0.5, 0.25, duration)
            self:CreateDamageEffects( ranBone, self:GetArmy() )
            self:CreateExplosionDebris( ranBone, self:GetArmy() )
            self:CreateFirePlumes( self:GetArmy(), {ranBone}, Random(0,2) )
            self:CreateAmmoCookOff( self:GetArmy(), {ranBone}, Random(0,2) )
            WaitSeconds( duration )
        end

    end,

    DeathThread = function( self, overkillRatio , instigator)
        # Create explosion effects
        self:InitialRandomExplosionsCybrans() 
        WaitSeconds(2)  
        explosion.CreateDefaultHitExplosionAtBone( self, 'Turret', 10.0 )
        self:PlayUnitSound('Destroyed')
        self:ShakeCamera(8, 1.5, 0.75, 3)
        CreateAttachedEmitter(self, 0, self:GetArmy(), '/effects/emitters/nuke_concussion_ring_01_emit.bp'):ScaleEmitter(0.175)
        CreateAttachedEmitter(self, 1, self:GetArmy(), '/effects/emitters/shockwave_smoke_01_emit.bp' )

        #Starts the corpse effects

        if( self.ShowUnitDestructionDebris and overkillRatio ) then
           if overkillRatio <= 1 then
               self.CreateUnitDestructionDebris( self, true, true, false )
           elseif overkillRatio <= 2 then
               self.CreateUnitDestructionDebris( self, true, true, false )
           elseif overkillRatio <= 3 then
               self.CreateUnitDestructionDebris( self, true, true, true )
           else #VAPORIZED
               self.CreateUnitDestructionDebris( self, true, true, true )
           end
        end

        self:CreateWreckage(1)
        WaitSeconds(8.0)
        self:DestroyAllDamageEffects()
        self:Destroy()
    end,
}

TypeClass = URL0403